@@ -26,7 +26,6 @@ from six import text_type
 from six.moves import http_cookiejar
 
 from webtest.compat import urlparse
-from webtest.compat import urlencode
 from webtest.compat import to_bytes
 from webtest.compat import escape_cookie_value
 from webtest.response import TestResponse
@@ -100,7 +99,7 @@ class TestApp(object):
 
         It can also be an actual full URL to an http server and webtest
         will proxy requests with `WSGIProxy2
-        <https://pypi.python.org/pypi/WSGIProxy2/>`_.
+        <https://pypi.org/project/WSGIProxy2/>`_.
     :type app:
         WSGI application
     :param extra_environ:
@@ -184,8 +183,12 @@ class TestApp(object):
         self.JSONEncoder = json_encoder
 
     def get_authorization(self):
-        """Allow to set the HTTP_AUTHORIZATION environ key. Value should looks
-        like ``('Basic', ('user', 'password'))``
+        """Allow to set the HTTP_AUTHORIZATION environ key. Value should look
+        like one of the following:
+
+        * ``('Basic', ('user', 'password'))``
+        * ``('Bearer', 'mytoken')``
+        * ``('JWT', 'myjwt')``
 
         If value is None the the HTTP_AUTHORIZATION is removed
         """
@@ -231,6 +234,10 @@ class TestApp(object):
         Sets a cookie to be passed through with requests.
 
         """
+        cookie_domain = self.extra_environ.get('HTTP_HOST', '.localhost')
+        cookie_domain = cookie_domain.split(':', 1)[0]
+        if '.' not in cookie_domain:
+            cookie_domain = "%s.local" % cookie_domain
         value = escape_cookie_value(value)
         cookie = http_cookiejar.Cookie(
             version=0,
@@ -238,7 +245,7 @@ class TestApp(object):
             value=value,
             port=None,
             port_specified=False,
-            domain='.localhost',
+            domain=cookie_domain,
             domain_specified=True,
             domain_initial_dot=False,
             path='/',
@@ -307,13 +314,7 @@ class TestApp(object):
         url = str(url)
         url = self._remove_fragment(url)
         if params:
-            if not isinstance(params, string_types):
-                params = urlencode(params, doseq=True)
-            if str('?') in url:
-                url += str('&')
-            else:
-                url += str('?')
-            url += params
+            url = utils.build_params(url, params)
         if str('?') in url:
             url, environ['QUERY_STRING'] = url.split(str('?'), 1)
         else:
@@ -333,18 +334,17 @@ class TestApp(object):
         Do a POST request. Similar to :meth:`~webtest.TestApp.get`.
 
         :param params:
-            Are put in the body of the request. If params is a
-            iterator it will be urlencoded, if it is string it will not
+            Are put in the body of the request. If params is an
+            iterator, it will be urlencoded. If it is a string, it will not
             be encoded, but placed in the body directly.
 
-            Can be a collections.OrderedDict with
+            Can be a :class:`python:collections.OrderedDict` with
             :class:`webtest.forms.Upload` fields included::
 
-
-            app.post('/myurl', collections.OrderedDict([
-                ('textfield1', 'value1'),
-                ('uploadfield', webapp.Upload('filename.txt', 'contents'),
-                ('textfield2', 'value2')])))
+                app.post('/myurl', collections.OrderedDict([
+                    ('textfield1', 'value1'),
+                    ('uploadfield', webapp.Upload('filename.txt', 'contents'),
+                    ('textfield2', 'value2')])))
 
         :param upload_files:
             It should be a list of ``(fieldname, filename, file_content)``.
@@ -441,7 +441,7 @@ class TestApp(object):
                                  upload_files=None,
                                  expect_errors=expect_errors)
 
-    def head(self, url, headers=None, extra_environ=None,
+    def head(self, url, params=None, headers=None, extra_environ=None,
              status=None, expect_errors=False, xhr=False):
         """
         Do a HEAD request. Similar to :meth:`~webtest.TestApp.get`.
@@ -449,6 +449,8 @@ class TestApp(object):
         :returns: :class:`webtest.TestResponse` instance.
 
         """
+        if params:
+            url = utils.build_params(url, params)
         if xhr:
             headers = self._add_xhr_header(headers)
         return self._gen_request('HEAD', url, headers=headers,
@@ -515,8 +517,15 @@ class TestApp(object):
                         file_info.append(value.content_type)
                 _append_file(file_info)
             else:
-                if isinstance(value, text_type):
+                if isinstance(value, int):
+                    value = str(value).encode('utf8')
+                elif isinstance(value, text_type):
                     value = value.encode('utf8')
+                elif not isinstance(value, (bytes, str)):
+                    raise ValueError((
+                        'Value for field {0} is a {1} ({2}). '
+                        'It must be str, bytes or an int'
+                    ).format(key, type(value), value))
                 lines.extend([
                     b'--' + boundary,
                     b'Content-Disposition: form-data; name="' + key + b'"',
